"""Construct and Return SQL Queries"""
from typing import Tuple, Dict

from ws_metrics.enums import FetchType
from ws_metrics.queries.query_definitions import Query


def determine_fetch(cursor, fetch_type: FetchType) -> Tuple:
    if fetch_type == FetchType.ONE:
        return cursor.fetchone()
    else:
        return cursor.fetchall()


class QueryString:
    def __init__(self, query_type: Query, conditionals: bool, index: int = None):
        self.query_name = query_type
        self.use_conditionals = conditionals
        self.index = index if index is not None else 0
        self.query = self.build_query_string(self.index)

    def build_query_string(self, conditional_index: int) -> str:
        base_str = self.query_name.select
        if self.use_conditionals:
            return (
                f"{base_str} {self.determine_from_clause(self.query_name)}"
                f" {self.determine_conditionals(self.query_name, conditional_index)}"
            )
        return f"{base_str} {self.determine_from_clause(self.query_name)}"

    def determine_from_clause(self, query_type: Query):
        return f"{query_type.from_clause} {self.determine_where_clause(query_type)}"

    @staticmethod
    def determine_where_clause(query_type: Query):
        return query_type.where

    @staticmethod
    def determine_conditionals(query_type: Query, index: int):
        if query_type.conditionals:
            return query_type.conditionals[index]
        # else query has no possible conditionals
        return ""


class QueryConstructor:
    def __init__(self, connection):
        self.conn = connection

    def construct_query(self, sql: QueryString, params: Dict, fetch_type: FetchType):
        print(f"Query Used: {sql.query}")
        cursor = self.conn.cursor()
        cursor.execute(sql.query, params)
        data_return = determine_fetch(cursor, fetch_type)
        return data_return
